/*
 * Daneel - Dalvik to Java bytecode compiler
 * Copyright (C) 2011  IcedRobot team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * This file is subject to the "Classpath" exception:
 *
 * Linking this library statically or dynamically with other modules is
 * making a combined work based on this library.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent
 * modules, and to copy and distribute the resulting executable under terms
 * of your choice, provided that you also meet, for each linked independent
 * module, the terms and conditions of the license of that module.  An
 * independent module is a module which is not derived from or based on
 * this library.  If you modify this library, you may extend this exception
 * to your version of the library, but you are not obligated to do so.  If
 * you do not wish to do so, delete this exception statement from your
 * version.
 */

package org.icedrobot.daneel.rewriter;

import java.util.Arrays;
import java.util.HashMap;

public class Interpreter {
    private final Register[] registers;
    private final HashMap<Object, Register[]> joinPointMap = new HashMap<Object, Register[]>();
    private boolean isDead;

    public Interpreter(int maxRegisters) {
        Register[] registers = new Register[maxRegisters];
        for (int i = 0; i < maxRegisters; i++) {
            registers[i] = new Register();
        }
        this.registers = registers;
    }

    public void load(int vregister, int expectedType) {
        registers[vregister].load(expectedType);
    }

    public void store(int vregister, int type) {
        registers[vregister].store(type);
    }

    public void storeArbitraryType(int vregister, int arbitraryType,
            Patchable patchable) {
        registers[vregister].storeArbitraryType(arbitraryType, patchable);
    }

    public void copy(int vdst, int vsrc) {
        registers[vdst].copy(registers[vsrc]);
    }

    public void storeJoinPoint(Object label) {
        joinPointMap.put(label, registers.clone());
    }

    public Register[] getJoinPoint(Object label) {
        return joinPointMap.get(label);
    }

    public void setDead() {
        this.isDead = true;
    }

    public boolean isDead() {
        return isDead;
    }

    public void merge(Register[] registers) {
        assert this.registers.length == registers.length;

        if (isDead) {
            System.arraycopy(registers, registers.length, this.registers, 0,
                    registers.length);
            isDead = false;
            return;
        }

        for (int i = 0; i < registers.length; i++) {
            this.registers[i].merge(registers[i]);
        }
    }

    @Override
    public String toString() {
        return Arrays.toString(registers);
    }
}
